Imports System
Imports System.Collections.Generic
Imports System.ComponentModel
Imports System.Data
Imports System.Drawing
Imports System.Text
Imports System.Windows.Forms
Imports EposPCmd.Net

Namespace Demo_WinDLL
    Partial Public Class Form1
        Inherits Form
#Region "private members"

        Private _updateActive As Boolean
        Private _deviceManager As DeviceManager
        Private _plc As Plc

#End Region

        ''' <summary>
        ''' Initializes a new instance of the <see cref="Form1"/> class.
        ''' </summary>
        Public Sub New()
            InitializeComponent()

            _updateActive = False
        End Sub

        ''' <summary>
        ''' Handles the Click event of the buttonExit control.
        ''' </summary>
        ''' <param name="sender">The source of the event.</param>
        ''' <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        Private Sub buttonExit_Click(ByVal sender As Object, ByVal e As EventArgs) Handles buttonExit.Click
            DialogResult = DialogResult.OK

            Close()
        End Sub

        ''' <summary>
        ''' Handles the Load event of the Form1 control.
        ''' </summary>
        ''' <param name="sender">The source of the event.</param>
        ''' <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        Private Sub Form1_Load(ByVal sender As Object, ByVal e As EventArgs) Handles MyBase.Load
            InitSettings(True)
        End Sub

        ''' <summary>
        ''' Handles the Click event of the buttonSettings control.
        ''' </summary>
        ''' <param name="sender">The source of the event.</param>
        ''' <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        Private Sub buttonSettings_Click(ByVal sender As Object, ByVal e As EventArgs) Handles buttonSettings.Click
            InitSettings(True)
        End Sub

        ''' <summary>
        ''' Starts the timer.
        ''' </summary>
        ''' <returns></returns>
        Private Function StartTimer() As Boolean
            If Not timerStatus.Enabled Then
                timerStatus.Interval = 1000
                timerStatus.Start()

                _updateActive = True
            End If

            Return True
        End Function

        ''' <summary>
        ''' Stops the timer.
        ''' </summary>
        ''' <returns></returns>
        Private Function StopTimer() As Boolean
            timerStatus.[Stop]()

            _updateActive = False

            Return True
        End Function


        ''' <summary>
        ''' Closes the settings.
        ''' </summary>
        ''' <returns></returns>
        Private Function CloseSettings() As Boolean
            If Not StopTimer() Then
                Return False
            End If

            If Not ClosePlcSettings() Then
                Return False
            End If

            Return CloseCommSettings()
        End Function

        ''' <summary>
        ''' Closes the PLC settings.
        ''' </summary>
        ''' <returns></returns>
        Private Function ClosePlcSettings() As Boolean
            Dim retVal As Boolean = False

            Try
                If _plc IsNot Nothing Then
                    _plc.Dispose()
                    _plc = Nothing
                End If
                retVal = True
            Catch e As Exception
                MessageBox.Show(e.Message, Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.[Error])
            End Try


            Return retVal
        End Function

        ''' <summary>
        ''' Closes the comm settings.
        ''' </summary>
        ''' <returns></returns>
        Private Function CloseCommSettings() As Boolean
            Dim retVal As Boolean = False

            Try
                If _deviceManager IsNot Nothing Then
                    _deviceManager.Dispose()
                    _deviceManager = Nothing
                End If

                retVal = True
            Catch e As Exception
                MessageBox.Show(e.Message, Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.[Error])
            End Try

            Return retVal
        End Function

        ''' <summary>
        ''' Opens the comm settings.
        ''' </summary>
        ''' <param name="showMsg">if set to <c>true</c> [show MSG].</param>
        ''' <returns></returns>
        Private Function OpenCommSettings(ByVal showMsg As Boolean) As Boolean
            Dim retVal As Boolean = False

            Try
                _deviceManager = New DeviceManager()

                retVal = True
            Catch e As Exception
                If showMsg Then
                    MessageBox.Show(e.Message, Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.[Error])
                End If
            End Try

            Return retVal
        End Function

        ''' <summary>
        ''' Opens the PLC settings.
        ''' </summary>
        ''' <param name="showMsg">if set to <c>true</c> [show MSG].</param>
        ''' <returns></returns>
        Private Function OpenPlcSettings(ByVal showMsg As Boolean) As Boolean
            Dim retVal As Boolean = False
            Dim deviceName As String = "EPOS2 P"

            Try
                _plc = _deviceManager.CreatePlc(deviceName)

                _plc.NodeId = [Byte].Parse(textBoxNodeId.Text)

                retVal = True
            Catch e As Exception
                If showMsg Then
                    MessageBox.Show(e.Message, Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.[Error])
                End If
            End Try

            Return retVal
        End Function

        ''' <summary>
        ''' Inits the settings.
        ''' </summary>
        ''' <param name="showMsg">if set to <c>true</c> [show MSG].</param>
        ''' <returns></returns>
        Private Function InitSettings(ByVal showMsg As Boolean) As Boolean
            If Not CloseSettings() Then
                Return False
            End If

            If Not OpenCommSettings(showMsg) Then
                Return False
            End If

            If Not OpenPlcSettings(showMsg) Then
                Return False
            End If

            Return StartTimer()
        End Function

        ''' <summary>
        ''' Writes the amplitude.
        ''' </summary>
        ''' <param name="amplitude">The amplitude.</param>
        ''' <returns></returns>
        Private Function WriteAmplitude(ByVal amplitude As UInteger) As Boolean
            Dim result As Boolean = False

            Try
                If _plc IsNot Nothing Then
                    Dim processInputType As EProcessInputType = EProcessInputType.PitProcessInputInt32
                    Dim elementNumber As Byte = 1
                    Dim data As Byte() = BitConverter.GetBytes(amplitude)
                    Dim nbOfBytesWritten As UInteger = 0

                    _plc.ProcessControl.ProcessInputOutput.SetProcessInput(processInputType, elementNumber, data, nbOfBytesWritten)
                    result = True
                End If
            Catch e As Exception
                result = False
                textBoxStatus.Text = e.Message
            End Try

            Return result
        End Function


        ''' <summary>
        ''' Updates the amplitude.
        ''' </summary>
        ''' <returns></returns>
        Private Function UpdateAmplitude() As Boolean
            Return WriteAmplitude(UInt32.Parse(textBoxAmplitude.Text))
        End Function

        ''' <summary>
        ''' Starts the cycles.
        ''' </summary>
        ''' <returns></returns>
        Private Function StartCycles() As Boolean
            Dim result As Boolean = False

            Try
                If _plc IsNot Nothing Then
                    Dim processInputType As EProcessInputType = EProcessInputType.PitProcessInputInt8
                    Dim elementNumber As Byte = 1
                    Dim bitNumber As Byte = 0
                    Dim bitState_Low As Byte = 0
                    Dim bitState_High As Byte = 1

                    _plc.ProcessControl.ProcessInputOutput.SetProcessInputBit(processInputType, elementNumber, bitNumber, bitState_Low)

                    _plc.ProcessControl.ProcessInputOutput.SetProcessInputBit(processInputType, elementNumber, bitNumber, bitState_High)
                    Return True
                End If
            Catch e As Exception
                result = False
                textBoxStatus.Text = e.Message
            End Try

            Return result
        End Function

        ''' <summary>
        ''' Stops the cycles.
        ''' </summary>
        ''' <returns></returns>
        Private Function StopCycles() As Boolean
            Dim result As Boolean = False

            Try
                If _plc IsNot Nothing Then
                    Dim processInputType As EProcessInputType = EProcessInputType.PitProcessInputInt8
                    Dim elementNumber As Byte = 1
                    Dim bitNumber As Byte = 0
                    Dim bitState_Low As Byte = 0
                    Dim bitState_High As Byte = 1

                    _plc.ProcessControl.ProcessInputOutput.SetProcessInputBit(processInputType, elementNumber, bitNumber, bitState_High)

                    _plc.ProcessControl.ProcessInputOutput.SetProcessInputBit(processInputType, elementNumber, bitNumber, bitState_Low)
                    result = True
                End If
            Catch e As Exception
                result = False
                textBoxStatus.Text = e.Message
            End Try

            Return result
        End Function

        ''' <summary>
        ''' Resets the error.
        ''' </summary>
        ''' <returns></returns>
        Private Function ResetError() As Boolean
            Dim result As Boolean = False

            Try
                If _plc IsNot Nothing Then
                    Dim processInputType As EProcessInputType = EProcessInputType.PitProcessInputInt8
                    Dim elementNumber As Byte = 1
                    Dim bitNumber As Byte = 1
                    Dim bitState_Low As Byte = 0
                    Dim bitState_High As Byte = 1

                    _plc.ProcessControl.ProcessInputOutput.SetProcessInputBit(processInputType, elementNumber, bitNumber, bitState_Low)

                    _plc.ProcessControl.ProcessInputOutput.SetProcessInputBit(processInputType, elementNumber, bitNumber, bitState_High)
                End If
            Catch e As Exception
                result = False
                textBoxStatus.Text = e.Message
            End Try

            Return result
        End Function

        ''' <summary>
        ''' Handles the Click event of the buttonStartCycles control.
        ''' </summary>
        ''' <param name="sender">The source of the event.</param>
        ''' <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        Private Sub buttonStartCycles_Click(ByVal sender As Object, ByVal e As EventArgs) Handles buttonStartCycles.Click
            If UpdateAmplitude() Then
                StartCycles()
            End If
        End Sub

        ''' <summary>
        ''' Handles the Click event of the buttonStopCycles control.
        ''' </summary>
        ''' <param name="sender">The source of the event.</param>
        ''' <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        Private Sub buttonStopCycles_Click(ByVal sender As Object, ByVal e As EventArgs) Handles buttonStopCycles.Click
            StopCycles()
        End Sub

        ''' <summary>
        ''' Handles the Click event of the buttonResetError control.
        ''' </summary>
        ''' <param name="sender">The source of the event.</param>
        ''' <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        Private Sub buttonResetError_Click(ByVal sender As Object, ByVal e As EventArgs) Handles buttonResetError.Click
            ResetError()
        End Sub

        ''' <summary>
        ''' Handles the Click event of the buttonDownloadProgram control.
        ''' </summary>
        ''' <param name="sender">The source of the event.</param>
        ''' <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        Private Sub buttonDownloadProgram_Click(ByVal sender As Object, ByVal e As EventArgs) Handles buttonDownloadProgram.Click
            Try
                Dim fileName As String

                Dim dlg As New OpenFileDialog()

                dlg.Filter = "Project files (*.var)|*.var|(*.mem)|*.mem|All files (*.*)|*.*"

                If dlg.ShowDialog() = DialogResult.OK Then
                    fileName = dlg.FileName

                    _plc.ProcessControl.ProgramControl.DownloadProgram(fileName)
                End If
            Catch ex As Exception
                MessageBox.Show(ex.Message, Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.[Error])
            End Try
        End Sub

        ''' <summary>
        ''' Handles the Click event of the buttonStartProgram control.
        ''' </summary>
        ''' <param name="sender">The source of the event.</param>
        ''' <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        Private Sub buttonStartProgram_Click(ByVal sender As Object, ByVal e As EventArgs) Handles buttonStartProgram.Click
            Try
                _plc.ProcessControl.ProgramControl.ColdstartPlc()
            Catch ex As Exception
                MessageBox.Show(ex.Message, Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.[Error])
            End Try
        End Sub

        ''' <summary>
        ''' Handles the Click event of the buttonStopProgram control.
        ''' </summary>
        ''' <param name="sender">The source of the event.</param>
        ''' <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        Private Sub buttonStopProgram_Click(ByVal sender As Object, ByVal e As EventArgs) Handles buttonStopProgram.Click
            Try
                _plc.ProcessControl.ProgramControl.StopPlc()
            Catch ex As Exception
                MessageBox.Show(ex.Message, Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.[Error])
            End Try
        End Sub

        ''' <summary>
        ''' Determines whether this instance is error.
        ''' </summary>
        ''' <returns>
        ''' 	<c>true</c> if this instance is error; otherwise, <c>false</c>.
        ''' </returns>
        Private Function IsError() As Boolean
            Dim result As Boolean = False

            Try
                If _plc IsNot Nothing Then
                    Dim processOutputType As EProcessOutputType = EProcessOutputType.PitProcessOutputInt8
                    Dim elementNumber As Byte = 1
                    Dim bitNumber As Byte = 1
                    Dim bitState As Byte = 0

                    _plc.ProcessControl.ProcessInputOutput.GetProcessOutputBit(processOutputType, elementNumber, bitNumber, bitState)

                    result = bitState > 0
                End If
            Catch e As Exception
                result = True
                textBoxStatus.Text = e.Message
            End Try

            Return result
        End Function

        ''' <summary>
        ''' Determines whether this instance is running.
        ''' </summary>
        ''' <returns>
        ''' 	<c>true</c> if this instance is running; otherwise, <c>false</c>.
        ''' </returns>
        Private Function IsRunning() As Boolean
            Dim result As Boolean = False

            Try
                If _plc IsNot Nothing Then
                    Dim processOutputType As EProcessOutputType = EProcessOutputType.PitProcessOutputInt8
                    Dim elementNumber As Byte = 1
                    Dim bitNumber As Byte = 0
                    Dim bitState As Byte = 0

                    _plc.ProcessControl.ProcessInputOutput.GetProcessOutputBit(processOutputType, elementNumber, bitNumber, bitState)

                    result = bitState > 0
                End If
            Catch e As Exception
                result = False
                textBoxStatus.Text = e.Message
            End Try

            Return result
        End Function

        ''' <summary>
        ''' Reads the count.
        ''' </summary>
        ''' <param name="count">The count.</param>
        ''' <returns></returns>
        Private Function ReadCount(ByRef count As UInteger) As Boolean
            Dim result As Boolean = True

            count = 0

            Try
                If _plc IsNot Nothing Then
                    Dim processOutputType As EProcessOutputType = EProcessOutputType.PitProcessOutputUint32
                    Dim elementNumber As Byte = 1
                    Dim data As Byte() = New Byte(4) {}
                    Dim nbOfBytesRead As UInteger = 0

                    _plc.ProcessControl.ProcessInputOutput.GetProcessOutput(processOutputType, elementNumber, data, nbOfBytesRead)

                    count = BitConverter.ToUInt32(data, 0)
                End If
            Catch e As Exception
                result = False
                textBoxStatus.Text = e.Message
            End Try

            Return result
        End Function

        ''' <summary>
        ''' Updates the status.
        ''' </summary>
        ''' <returns></returns>
        Private Function UpdateStatus() As Boolean
            Dim retVal As Boolean = True

            Try
                SuspendLayout()

                If _plc IsNot Nothing AndAlso _updateActive Then
                    Dim isRunning__1 As Integer = 0
                    Dim isProgramAvailable As Integer = 0

                    _plc.ProcessControl.ProgramControl.GetPlcStatus(isRunning__1, isProgramAvailable)

                    If isProgramAvailable <> 0 Then
                        textBoxStatus.Text = "Program valid"
                    Else
                        textBoxStatus.Text = "No program"
                    End If

                    buttonDownloadProgram.Enabled = (isProgramAvailable = 0 OrElse isRunning__1 = 0)

                    If isProgramAvailable <> 0 Then

                        If isRunning__1 <> 0 Then
                            textBoxStatus.Text = "Running"
                        Else
                            textBoxStatus.Text = "Not running"
                        End If


                        If isRunning__1 <> 0 Then
                            buttonSettings.Enabled = False
                            textBoxNodeId.Enabled = False
                            buttonStopProgram.Enabled = True
                            buttonStartProgram.Enabled = False

                            If IsError() Then
                                textBoxStatus.Text = "Cycle error"
                                buttonResetError.Enabled = True
                                buttonStopCycles.Enabled = False
                                buttonStartCycles.Enabled = False
                            ElseIf IsRunning() Then
                                textBoxStatus.Text = "Cycle running"
                                buttonStopProgram.Enabled = True
                                buttonStopCycles.Enabled = True
                                buttonStartCycles.Enabled = False
                            Else
                                textBoxStatus.Text = "Cycle stopped"
                                buttonStartProgram.Enabled = False
                                buttonStopProgram.Enabled = True
                                buttonStartCycles.Enabled = True
                                buttonStopCycles.Enabled = False
                            End If
                        Else
                            buttonStartProgram.Enabled = True
                            buttonStopProgram.Enabled = False
                            buttonStartCycles.Enabled = False
                            buttonStopCycles.Enabled = False
                            buttonResetError.Enabled = False
                            buttonSettings.Enabled = True
                            textBoxNodeId.Enabled = True
                            textBoxAmplitude.Enabled = False
                        End If
                    End If

                    Dim count As UInteger = 0

                    If Not ReadCount(count) Then
                        textBoxCount.Text = "0"
                    Else
                        textBoxCount.Text = [String].Format("{0}", count)
                    End If
                Else
                    NoCommunication()
                End If
            Catch e As Exception
                retVal = False

                NoCommunication()
            Finally
                ResumeLayout()
            End Try

            Return retVal
        End Function

        ''' <summary>
        ''' Noes the communication.
        ''' </summary>
        Private Sub NoCommunication()
            textBoxStatus.Text = "No communication"

            buttonStartProgram.Enabled = False
            buttonStopProgram.Enabled = False
            buttonStartCycles.Enabled = False
            buttonStopCycles.Enabled = False
            buttonResetError.Enabled = False
            buttonSettings.Enabled = True
            textBoxNodeId.Enabled = True
            textBoxAmplitude.Enabled = False
        End Sub

        ''' <summary>
        ''' Handles the Tick event of the timerStatus control.
        ''' </summary>
        ''' <param name="sender">The source of the event.</param>
        ''' <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        Private Sub timerStatus_Tick(ByVal sender As Object, ByVal e As EventArgs) Handles timerStatus.Tick
            If _updateActive AndAlso Not UpdateStatus() Then
                StopTimer()
            End If
        End Sub

        ''' <summary>
        ''' Handles the TextChanged event of the textBoxAmplitude control.
        ''' </summary>
        ''' <param name="sender">The source of the event.</param>
        ''' <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        Private Sub textBoxAmplitude_TextChanged(ByVal sender As Object, ByVal e As EventArgs) Handles textBoxAmplitude.TextChanged
            UpdateAmplitude()
        End Sub

        ''' <summary>
        ''' Handles the TextChanged event of the textBoxNodeId control.
        ''' </summary>
        ''' <param name="sender">The source of the event.</param>
        ''' <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        Private Sub textBoxNodeId_TextChanged(ByVal sender As Object, ByVal e As EventArgs) Handles textBoxNodeId.TextChanged
            Try
                If _plc IsNot Nothing Then
                    _plc.NodeId = Convert.ToByte(textBoxNodeId.Text)

                    StartTimer()
                End If
            Catch ex As Exception
                textBoxStatus.Text = ex.Message
            End Try
        End Sub

        ''' <summary>
        ''' Handles the FormClosing event of the Form1 control.
        ''' </summary>
        ''' <param name="sender">The source of the event.</param>
        ''' <param name="e">The <see cref="System.Windows.Forms.FormClosingEventArgs"/> instance containing the event data.</param>
        Private Sub Form1_FormClosing(ByVal sender As Object, ByVal e As FormClosingEventArgs) Handles MyBase.FormClosing
            CloseSettings()
        End Sub


    End Class
End Namespace
